@@ -7,11 +7,9 @@ from .config import (
     DsnConnection,
     Schema,
     Field,
-    ObjectField,
-    JsonField,
     Index
 )
-from .query import Query, CacheQuery
+from .query import Query
 from . import decorators
 from .model import Orm
 from .interface import (
@@ -25,7 +23,7 @@ from .exception import InterfaceError, Error, UniqueError
 from . import utils
 
 
-__version__ = '1.4.4'
+__version__ = '2.1.0'
 
 
 # get rid of "No handler found" warnings (cribbed from requests)
