@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals, division, print_function, absolute_import
+import decimal
 import datetime
 import inspect
 import re
@@ -49,12 +50,6 @@ class Connection(object):
     options = None
     """any other db options, these can be interface implementation specific"""
 
-#     @property
-#     def interface_class(self):
-#         """Return the configured interface class object that can be used to create new instances"""
-#         interface_module, interface_class = utils.get_objects(self.interface_name)
-#         return interface_class
-
     @property
     def interface(self):
         """Return a new Interface instance using this configuration"""
@@ -65,7 +60,7 @@ class Connection(object):
         """
         set all the values by passing them into this constructor, any unrecognized kwargs get put into .options
 
-        example --
+        :example:
             c = Connection(
                 host="127.0.0.1",
                 database="dbname",
@@ -73,10 +68,9 @@ class Connection(object):
                 some_random_thing="foo"
             )
 
-            print c.port # 5000
-            print c.options # {"some_random_thing": "foo"}
+            print(c.port) # 5000
+            print(c.options) # {"some_random_thing": "foo"}
         """
-
         kwargs = utils.make_dict(fields, fields_kwargs)
         if "interface_name" not in kwargs:
             raise ValueError("no interface_name passed into Connection")
@@ -89,7 +83,6 @@ class Connection(object):
             else:
                 self.options[key] = val
 
-
         interface_module, interface_class = utils.get_objects(self.interface_name)
         self.interface_class = interface_class
         interface_class.configure(self)
@@ -103,10 +96,11 @@ class DsnConnection(Connection):
 
         <full.python.path.InterfaceClass>://<username>:<password>@<host>:<port>/<database>?<options=val&query=string>#<name>
 
-    This is useful to allow connections coming in through environment variables as described
-    http://www.12factor.net/backing-services
+    This is useful to allow connections coming in through environment variables as described in :
+        http://www.12factor.net/backing-services
 
-    It tooks its inspiration from this project https://github.com/kennethreitz/dj-database-url
+    It tooks its inspiration from this project:
+        https://github.com/kennethreitz/dj-database-url
 
     http://en.wikipedia.org/wiki/Connection_string
     http://en.wikipedia.org/wiki/Data_source_name
@@ -183,7 +177,7 @@ class Schema(object):
     instances = {}
     """class variable, holds different schema instances for various orms"""
 
-    table_name = u""
+    table_name = ""
     """string -- set the table name for this schema instance"""
 
     fields = None
@@ -192,6 +186,9 @@ class Schema(object):
     indexes = None
     """dict -- all the indexes this schema will have"""
 
+    lookup = None
+    """dict -- field information lookup table, basically an internal cache"""
+
     @property
     def normal_fields(self):
         """fields that aren't magic (eg, don't start with an underscore)"""
@@ -217,24 +214,39 @@ class Schema(object):
         """returns the primary key name for this schema"""
         try:
             pk_field = self.__getattr__("pk")
-            return pk_field.name
-
+            pk_name = pk_field.name
         except AttributeError:
-            pk_field = None
-            return None
+            pk_name = None
+        return pk_name
 
     @classmethod
     def get_instance(cls, orm_class):
+        """return a Schema singleton instance for the given orm_class
+
+        if there isn't already an instance in cache then a new instance will be
+        created. If a Schema instance is already in cache then it will be returned
+
+        :param orm_class: Orm, the class to create the schema for
+        :returns: Schema
+        """
         table_name = orm_class.table_name
         if table_name not in cls.instances:
             s = cls(table_name)
+            s.orm_class = orm_class
+
             seen_properties = set()
             for klass in inspect.getmro(orm_class)[:-1]:
                 for k, v in vars(klass).items():
                     if k not in seen_properties:
                         if isinstance(v, (Field, Index)):
-                            v.orm_class = orm_class
                             s.set(k, v)
+
+                        elif isinstance(v, type):
+                            # We've defined a Field class inline of the Orm, so
+                            # we want to instantiate it
+                            if issubclass(v, Field):
+                                s.set(k, v(v.type, v.required, v.options))
+
                         seen_properties.add(k)
 
             #s.orm_class = orm_class
@@ -242,18 +254,16 @@ class Schema(object):
 
         else:
             s = cls.instances[table_name]
+            s.orm_class = orm_class
 
-        s.orm_class = orm_class
         return s
 
     def __init__(self, table_name, **fields_or_indexes):
-        """
-        create an instance
+        """Create an instance
 
         every Orm should have a .schema attribute that is an instance of this class
 
-        example --
-
+        :example:
             schema = Schema(
                 "table_name"
                 field1=Field(int, True),
@@ -261,12 +271,18 @@ class Schema(object):
                 index_fields=Index("field1", "field2")
             )
 
-        table_name -- string -- the table name
-        **fields_or_indexes -- a dict of field name or index keys with tuple values, see __getattr__ for more details
+        :param table_name: string, the table name
+        :param **fields_or_indexes: a dict of field name or index keys with tuple values,
+            see __getattr__ for more details
         """
         self.fields = {}
         self.indexes = {}
-        self.table_name = str(table_name)
+        self.table_name = String(table_name)
+        self.orm_class = None
+        self.lookup = {
+            "names": {},
+            "pk": None,
+        }
 
         for name, val in fields_or_indexes.items():
             self.set(name, val)
@@ -297,15 +313,10 @@ class Schema(object):
 
         return -- string -- the string value of the attribute name, eg, self.foo returns "foo"
         """
-        if name in self.fields:
-            return self.fields[name]
-
-        else:
-            if name == "pk":
-                for field_name, field in self.fields.items():
-                    if field.options.get('pk', False):
-                        return field
+        try:
+            return self.lookup["names"][name]
 
+        except KeyError:
             raise AttributeError("No {} field in schema {}".format(name, self.table_name))
 
     def set_field(self, field_name, field):
@@ -314,10 +325,20 @@ class Schema(object):
         if not isinstance(field, Field): raise ValueError("{} is not a Field instance".format(type(field)))
 
         field.name = field_name
+        field.orm_class = self.orm_class
+
         if field.options['unique']:
             self.set_index(field_name, Index(field_name, unique=True))
 
         self.fields[field_name] = field
+
+        for fn in field.names:
+            self.lookup["names"][fn] = field
+
+        if field.is_pk():
+            self.lookup["pk"] = field
+            self.lookup["names"]["pk"] = field
+
         return self
 
     def set_index(self, index_name, index):
@@ -339,6 +360,8 @@ class Schema(object):
         if not isinstance(index, Index): raise ValueError("{} is not an Index instance".format(type(index)))
 
         index.name = index_name
+        index.orm_class = self.orm_class
+
         self.indexes[index_name] = index
         return self
 
@@ -351,6 +374,46 @@ class Schema(object):
         """
         return self.__getattr__(k).name
 
+    def create_orm(self, orm_class=None):
+        """If you have a schema but don't have an Orm for it, you can call this method
+        and have an orm_class created that will have the fields and table_name of this
+        schema
+
+        :param orm_class: Orm, if you want your generated class to have a certain
+            parent class you can pass in the parent class you want
+        :returns: Orm, your child orm populated with information from this schema
+        """
+        if not orm_class:
+            from .model import Orm # avoid circular dependency
+            orm_class = Orm
+
+        child_class = type(
+            ByteString(self.table_name) if is_py2 else String(self.table_name),
+            (orm_class,),
+            {"table_name": self.table_name, "schema": self}
+        )
+
+        # clear all the Field and Index properties in anticipation of adding the
+        # fields from the Schema
+        seen_properties = set()
+        for klass in inspect.getmro(child_class):
+            for k, v in vars(klass).items():
+                if k not in seen_properties:
+                    if isinstance(v, (Field, Index)):
+                        #setattr(child_class, k, property(lambda: AttributeError))
+                        setattr(child_class, k, None)
+                        #delattr(child_class, k)
+
+                    elif isinstance(v, type) and issubclass(v, Field):
+                        setattr(child_class, k, None)
+
+                    seen_properties.add(k)
+
+        for field_name, field in self:
+            setattr(child_class, field_name, field)
+
+        return child_class
+
 
 class Index(object):
     """Each index on the table is configured using this class"""
@@ -396,7 +459,7 @@ class Field(object):
             return val
 
         @foo.isetter
-        def foo(self, val, is_update, is_modified):
+        def foo(self, val):
             # do custom things
             return val
 
@@ -405,7 +468,7 @@ class Field(object):
     value
 
     There are also fget/fset/fdel methods that can be set to customize behavior
-    on when a value is set on a particular instance, so if you wanted to make sure
+    on when a value is set on a particular orm instance, so if you wanted to make sure
     that bar was always an int when it is set, you could:
 
         bar = Field(int, True)
@@ -427,42 +490,74 @@ class Field(object):
     NOTE -- the fget/fset/fdel methods are different than traditional python getters
     and setters because they always need to return a value and they always take in a
     value
+
+    https://docs.python.org/2/howto/descriptor.html
     """
+    type = None
+    """The python type of this field (eg, str, int, Orm)"""
+
+    required = False
+    """True if this field is required"""
+
+    options = None
+    """In the instance, this will be a dict of key/val pairs containing extra information
+    about the field"""
+
     @property
     def schema(self):
-        """return the schema instance if this is reference to another table"""
+        """return the schema instance if this is reference to another table
+
+        see .set_type() for an explanation on why we defer figuring this out until now
+        """
         if not hasattr(self, "_schema"):
-            ret = None
-            o = self._type
-            if isinstance(o, type):
-                ret = getattr(o, "schema", None)
+            field_type = self.original_type
+            module, klass = utils.get_objects(field_type)
+            schema = klass.schema
+            if not schema:
+                raise ValueError("Field type {} is not an Orm class".format(field_type))
 
-            elif isinstance(o, Schema):
-                ret = o
-
-            else:
-                module, klass = utils.get_objects(o)
-                ret = klass.schema
-
-            self._schema = ret
+            self._schema = schema
 
         return self._schema
 
     @property
     def type(self):
-        ret = self._type
-        if not isinstance(ret, type) or hasattr(ret, "schema"):
+        """Return the actual type this field should be
+
+        see .set_type() for an explanation on why we defer this until here
+        """
+        ret = self.original_type
+        if self.is_serialized():
+            ret = str
+
+        else:
             s = self.schema
-            ret = s.pk.type
+            if s:
+                ret = s.pk.type
 
         return ret
 
+    @property
+    def interface_type(self):
+        """Returns the type that will be used in the interface to create the table"""
+        return self.type
+
     @property
     def ref(self):
         """Returns the FK reference orm class"""
         schema = self.schema
         return schema.orm_class if schema else None
 
+    @property
+    def names(self):
+        names = []
+        if self.name:
+            names.append(self.name)
+
+        names.extend(self.options.get("names", []))
+        names.extend(self.options.get("aliases", []))
+        return names
+
     def __init__(self, field_type, field_required=False, field_options=None, **field_options_kwargs):
         """
         create a field
@@ -482,6 +577,31 @@ class Field(object):
         :param **field_options_kwargs: dict, will be combined with field_options
         """
         field_options = utils.make_dict(field_options, field_options_kwargs)
+
+        d = self.get_size(field_options)
+        field_options.update(d)
+
+        field_options.setdefault("unique", False)
+
+        self.name = field_options.pop("name", "")
+        # this creates a numeric dict key that can't be accessed as an attribute
+        self.instance_field_name = "{}{}".format(self.name, id(self))
+        self.default = field_options.pop("default", None)
+        self.orm_class = field_options.pop("orm_class", None)
+        self.help = field_options.pop("help", "")
+        self.choices = field_options.pop("choices", set())
+
+        self.set_type(field_type, field_options)
+
+        for k in list(field_options.keys()):
+            if hasattr(self, k):
+                setattr(self, k, field_options.pop(k))
+
+        self.options = field_options
+        self.required = field_required or self.is_pk()
+
+    def get_size(self, field_options):
+        """Figure out if this field has any size information"""
         d = {}
 
         min_size = field_options.pop("min_size", -1)
@@ -501,34 +621,77 @@ class Field(object):
                 d['min_size'] = min_size
                 d['max_size'] = max_size
 
-        field_options.setdefault("unique", False)
-        field_options.update(d)
+        return d
 
-        self.fgetter(field_options.pop("fget", self.default_fget))
-        self.fsetter(field_options.pop("fset", self.default_fset))
-        self.fdeleter(field_options.pop("fdel", self.default_fdel))
-        self.fdefaulter(field_options.pop("fdefault", self.default_fdefault))
+    def set_type(self, field_type, field_options):
+        """Try to infer as much about the type as can be inferred at this moment
 
-        self.igetter(field_options.pop("iget", self.default_iget))
-        self.isaver(field_options.pop("isave", self.default_isave))
-        self.imodifier(field_options.pop("imodify", self.default_imodify))
-        self.imodify_inserter(field_options.pop("imodify_insert", self.default_imodify_insert))
-        self.imodify_updater(field_options.pop("imodify_update", self.default_imodify_update))
-        self.idefaulter(field_options.pop("idefault", self.default_idefault))
-        self.idefault_inserter(field_options.pop("idefault_insert", self.default_idefault_insert))
-        self.idefault_updater(field_options.pop("idefault_update", self.default_idefault_update))
-        self.iquerier(field_options.pop("iquery", self.default_iquery))
+        Because the Field support string classpaths (eg, "modname.Classname") we can't
+        figure everything out in this method, so we figure out as much as we can
+        and then defer everything else to the .type and .schema properties, this
+        allows the parser to hopefully finish loading the modules before we have to
+        parse the classpath to find the foreign key schema
 
-        self.jsonabler(field_options.pop("jsonable", self.default_jsonable))
+        :param field_type: mixed, the field type passed into __init__
+        :param field_options: dict, the options passed into __init__
+        """
+        std_types = (
+            bool,
+            long,
+            int,
+            float,
+            bytearray,
+            decimal.Decimal,
+            datetime.datetime,
+            datetime.date,
+        )
+        if is_py2:
+            std_types = (basestring,) + std_types
+        else:
+            std_types = basestring + std_types
 
-        self.name = field_options.pop("name", "")
-        # this creates a numeric dict key that can't be accessed as an attribute
-        self.instance_field_name = str(id(self))
-        self._type = field_type
-        self.default = field_options.pop("default", None)
-        self.options = field_options
-        self.required = field_required if field_required else self.is_pk()
-        self.orm_class = None
+        json_types = (
+            dict,
+            list,
+        )
+
+        pickle_types = (
+            set,
+        )
+
+        self.original_type = field_type
+        self.serializer = ""
+
+        if isinstance(field_type, type):
+            if issubclass(field_type, std_types):
+                self._schema = None
+
+            elif issubclass(field_type, json_types):
+                self._schema = None
+                self.serializer = field_options.pop("serializer", "json")
+
+            elif issubclass(field_type, pickle_types):
+                self._schema = None
+                self.serializer = field_options.pop("serializer", "pickle")
+
+            else:
+                schema = getattr(field_type, "schema", None)
+                if schema:
+                    self._schema = schema
+
+                else:
+                    # We have just some random class that isn't an Orm
+                    self.serializer = field_options.pop("serializer", "pickle")
+                    self._schema = None
+
+        elif isinstance(field_type, Schema):
+            self._schema = field_type
+
+        else:
+            # check if field_type is a string classpath so we have to defer
+            # setting the type
+            if not isinstance(field_type, basestring):
+                raise ValueError("Unknown field type {}".format(field_type))
 
     def is_pk(self):
         """return True if this field is a primary key"""
@@ -539,18 +702,105 @@ class Field(object):
         return bool(self.schema)
 
     def is_required(self):
+        """Return True if this field is required to save into the interface"""
         return self.required
 
-    def default_fget(self, instance, val):
-        return self.fdefault(instance, val)
+    def is_serialized(self):
+        """Return True if this field should be serialized"""
+        return True if self.serializer else False
 
-    def default_fset(self, instance, val):
+    def fget(self, orm, val):
+        """Called anytime the field is accessed through the Orm (eg, Orm.foo)
+
+        :param orm: Orm, the Orm instance the field is being accessed on
+        :param val: mixed, the current value of the field
+        :returns: mixed
+        """
         return val
 
-    def default_fdel(self, instance, val):
+    def fgetter(self, v):
+        """decorator for setting field's fget function"""
+        self.fget = v
+        return self
+
+    def iget(self, orm, val):
+        """Called anytime the field is being returned from the interface to the orm
+
+        think of this as when the orm receives the field value from the interface
+
+        :param orm: Orm
+        :param val: mixed, the current value of the field
+        :returns: mixed
+        """
+        if self.is_serialized():
+            val = self.decode(val)
+        return val
+
+    def igetter(self, v):
+        """decorator for the method called when a field is pulled from the database"""
+        self.iget = v
+        return self
+
+    def fset(self, orm, val):
+        """This is called on Orm instantiation and any time field is set (eg Orm.foo = ...)
+
+        on Orm creation val will be None if the field wasn't pass to Orm.__init__ otherwise
+        it will be the value passed into Orm.__init__
+
+        :param orm: Orm, the Orm instance the field is being set on
+        :param val: mixed, the current value of the field
+        :returns: mixed
+        """
+        if val is not None and self.choices:
+            if val not in self.choices:
+                raise ValueError("Value {} not in {} value choices".format(val, self.name))
+        return val
+
+    def fsetter(self, v):
+        """decorator for setting field's fset function"""
+        self.fset = v
+        return self
+
+    def iset(self, orm, val):
+        """Called anytime the field is being fetched to send to the interface
+
+        think of this as when the interface is going to get the field value
+
+        :param orm: Orm
+        :param val: mixed, the current value of the field
+        :returns: mixed
+        """
+        if self.is_serialized():
+            val = self.encode(val)
+        return val
+
+    def isetter(self, v):
+        """decorator for setting field's fset function"""
+        self.iset = v
+        return self
+
+
+    def fdel(self, orm, val):
         return None
 
-    def default_fdefault(self, instance, val):
+    def fdeleter(self, v):
+        """decorator for setting field's fdel function"""
+        self.fdel = v
+        return self
+
+    def fdefault(self, orm, val):
+        """On a new Orm instantiation, this will be called with val=None and will
+        if val=None then this will decide how to use self.default to set the default
+        value of the field
+
+        usually you won't need to override this method because you can just pass
+        default into the field instantiation and it will get automatically used
+        in this method
+
+        :param orm: Orm, the Orm instance being created
+        :param val: mixed, the current value of the field (usually None)
+        :returns: mixed
+        """
         ret = val
         if val is None:
             if callable(self.default):
@@ -559,7 +809,7 @@ class Field(object):
             elif self.default is None:
                 ret = self.default
 
-            elif isinstance(self.default, (dict, list, set, object)):
+            elif isinstance(self.default, (dict, list, set)):
                 ret = type(self.default)()
 
             else:
@@ -567,52 +817,34 @@ class Field(object):
 
         return ret
 
-    def default_isave(self, instance, val, is_update, is_modified):
-        if is_modified:
-            val = self.imodify(instance, val)
-            if is_update:
-                val = self.imodify_update(instance, val)
+    def fdefaulter(self, v):
+        """decorator for returning the field's default value
 
-            else:
-                val = self.imodify_insert(instance, val)
+        decorator for setting field's fdefault function"""
+        self.fdefault = v
+        return self
 
-        else:
-            val = self.idefault(instance, val)
-            if is_update:
-                val = self.idefault_update(instance, val)
+    def iquery(self, query, val):
+        """This will be called when setting the field onto a query instance
 
-            else:
-                val = self.idefault_insert(instance, val)
+        :example:
+            o = Orm(foo=1)
+            o.query.eq_foo(1) # iquery called here
 
+        :param query: Query
+        :param val: mixed, the fields value
+        :returns: mixed
+        """
         return val
 
-    def default_idefault(self, instance, val):
-        return val
+    def iquerier(self, v):
+        """decorator for the method called when this field is used in a SELECT query"""
+        self.iquery = v
+        return self
 
-    def default_idefault_insert(self, instance, val):
-        return val
-
-    def default_idefault_update(self, instance, val):
-        return val
-
-    def default_imodify(self, instance, val):
-        return val
-
-    def default_imodify_insert(self, instance, val):
-        return val
-
-    def default_imodify_update(self, instance, val):
-        return val
-
-    def default_iget(self, instance, val):
-        return self.fdefault(instance, val)
-
-    def default_iquery(self, instance, val):
-        return val
-
-    def default_jsonable(self, instance, val):
+    def jsonable(self, orm, val):
         if val is None:
-            val = self.fdefault(instance, val)
+            val = self.fdefault(orm, val)
 
         if val is not None:
             format_str = ""
@@ -663,194 +895,80 @@ class Field(object):
 
         return val
 
-    def fgetter(self, v):
-        """decorator for setting field's fget function"""
-        self.fget = v
-        return self
-
-    def fsetter(self, v):
-        """decorator for setting field's fset function"""
-        self.fset = v
-        return self
-
-    def fdeleter(self, v):
-        """decorator for setting field's fdel function"""
-        self.fdel = v
-        return self
-
-    def fdefaulter(self, v):
-        """decorator for returning the field's default value
-
-        decorator for setting field's fdefault function"""
-        self.fdefault = v
-        return self
-
-    def igetter(self, v):
-        """decorator for the method called when a field is pulled from the database"""
-        self.iget = v
-        return self
-
-    def isaver(self, v):
-        """decorator for the method called when a field is saved into the database"""
-        self.isave = v
-        return self
-
-    def imodifier(self, v):
-        """decorator for the method called when an insert/update database query is going
-        to be used and that field has been modified"""
-        self.imodify = v
-        return self
-
-    def imodify_inserter(self, v):
-        """decorator for the method called when an insert database query is going
-        to be used and that field has been modified"""
-        self.imodify_insert = v
-        return self
-
-    def imodify_updater(self, v):
-        """decorator for the method called when an update database query is going
-        to be used and that field has been modified"""
-        self.imodify_update = v
-        return self
-
-    def idefaulter(self, v):
-        """decorator for the method called when an update/insert database query is going
-        to be used and that field hasn't been touched"""
-        self.idefault = v
-        return self
-
-    def idefault_inserter(self, v):
-        """decorator for the method called when an update/insert database query is going
-        to be used and that field hasn't been touched"""
-        self.idefault_insert = v
-        return self
-
-    def idefault_updater(self, v):
-        """decorator for the method called when an update/insert database query is going
-        to be used and that field hasn't been touched"""
-        self.idefault_update = v
-        return self
-
-    def iquerier(self, v):
-        """decorator for the method called when this field is used in a SELECT query"""
-        self.iquery = v
-        return self
-
     def jsonabler(self, v):
         """Decorator for the method called for a field when an Orm's .jsonable method
         is called"""
         self.jsonable = v
+        return self
 
-    def fval(self, instance):
-        """return the raw value that this property is holding internally for instance"""
+    def encode(self, val):
+        if val is None: return val
+
+        if self.serializer == "pickle":
+            return base64.b64encode(pickle.dumps(val, pickle.HIGHEST_PROTOCOL))
+
+        elif self.serializer == "json":
+            return json.dumps(val)
+
+        else:
+            raise ValueError("Unknown serializer {}".format(self.serializer))
+
+    def decode(self, val):
+        if val is None: return val
+
+        if self.serializer == "pickle":
+            return pickle.loads(base64.b64decode(val))
+
+        elif self.serializer == "json":
+            return json.loads(val)
+
+        else:
+            raise ValueError("Unknown serializer {}".format(self.serializer))
+
+    def fval(self, orm):
+        """return the raw value that this property is holding internally for the orm instance"""
         try:
-            val = instance.__dict__[self.instance_field_name]
+            val = orm.__dict__[self.instance_field_name]
         except KeyError as e:
             #raise AttributeError(str(e))
             val = None
 
         return val
 
-    def __get__(self, instance, classtype=None):
+    def __get__(self, orm, classtype=None):
         """This is the wrapper that will actually be called when the field is
         fetched from the instance, this is a little different than Python's built-in
         @property fget method because it will pull the value from a shadow variable in
         the instance and then call fget"""
-        if instance is None:
+        if orm is None:
             # class is requesting this property, so return it
             return self
 
-        raw_val = self.fval(instance)
-        ret = self.fget(instance, raw_val)
+        raw_val = self.fval(orm)
+        ret = self.fget(orm, raw_val)
 
         # we want to compensate for default values right here, so if the raw val
         # is None but the new val is not then we save the returned value, this
         # allows us to handle things like dict with no surprises
         if raw_val is None:
             if ret is not None:
-                instance.__dict__[self.instance_field_name] = ret
+                orm.__dict__[self.instance_field_name] = ret
 
         return ret
 
-    def __set__(self, instance, val):
+    def __set__(self, orm, val):
         """this is the wrapper that will actually be called when the field is
         set on the instance, your fset method must return the value you want set,
         this is different than Python's built-in @property setter because the
         fset method *NEEDS* to return something"""
-        val = self.fset(instance, val)
-        instance.__dict__[self.instance_field_name] = val
+        val = self.fset(orm, val)
+        orm.__dict__[self.instance_field_name] = val
 
-    def __delete__(self, instance):
+    def __delete__(self, orm):
         """the wrapper for when the field is deleted, for the most part the default
         fdel will almost never be messed with, this is different than Python's built-in
         @property deleter because the fdel method *NEEDS* to return something and it
         accepts the current value as an argument"""
-        val = self.fdel(instance, self.fval(instance))
-        instance.__dict__[self.instance_field_name] = val
-        #self.__set__(instance, val)
-
-
-class ObjectField(Field):
-    """A special field type for when you just want to shove an object in a field
-
-    this will just pickle and base64 the object so it can be stored in a text field
-    and then it will do the opposite when you pull it out, so basically, you can dump
-    anything you want in this field and it will be saved and restored transparently
-
-    I thought about doing Field(object, ...) but building it in that way actually
-    proved to be more complicated than I thought, you could pass in some type and if it
-    was that type then it would set the default methods to pickle_iset/iget, but I couldn't
-    decide what type to pass in, if you pass in pickle, then you need to check for pickle and
-    cPickle to decide, you can't pass in something like `object` without complicating 
-    how foreign keys are figured out, so ultimately, I've decided to just have it be
-    a separate class"""
-    def __init__(self, field_required=False, default=None):
-        """
-        unlike the normal Field class, you can't set any options or a type on this
-        Field, because it is a pickled object, so it can't be unique, it doesn't have
-        a size, etc.. Likewise, the field type is always str
-        """
-        super(ObjectField, self).__init__(
-            field_type=str,
-            field_required=field_required,
-            default=default,
-        )
-
-    def encode(self, val):
-        if val is None: return val
-        return base64.b64encode(pickle.dumps(val, pickle.HIGHEST_PROTOCOL))
-
-    def decode(self, val):
-        if val is None: return val
-        return pickle.loads(base64.b64decode(val))
-
-    def isaver(self, isave):
-        def master_isave(cls, val, is_update, is_modified):
-            v = isave(cls, val, is_update, is_modified)
-            return self.encode(val)
-            #return iset(cls, v, is_update, is_modified)
-        return super(ObjectField, self).isaver(master_isave)
-
-    def igetter(self, iget):
-        def master_iget(cls, val):
-            v = self.decode(val)
-            return iget(cls, v)
-        return super(ObjectField, self).igetter(master_iget)
-
-
-class JsonField(ObjectField):
-    """Similar to ObjectField but stores json in the db"""
-    def __init__(self, field_required=False, default=dict):
-        super(JsonField, self).__init__(
-            field_required=field_required,
-            default=default,
-        )
-
-    def encode(self, val):
-        if val is None: return val
-        return json.dumps(val)
-
-    def decode(self, val):
-        if val is None: return val
-        return json.loads(val)
+        val = self.fdel(orm, self.fval(orm))
+        orm.__dict__[self.instance_field_name] = val
 
