@@ -4,15 +4,17 @@ import inspect
 import sys
 import datetime
 
+from datatypes.collections import Pool
+
 # first party
 from .query import Query, Iterator
 from . import decorators, utils
 from .interface import get_interface
-from .config import Schema, Field, ObjectField, Index
+from .config import Schema, Field, Index
 from .compat import *
 
 
-class OrmPool(utils.Pool):
+class OrmPool(Pool):
     """
     Create a pool of Orm instances, which is just a dict of primary_key -> Orm instance
     mappings
@@ -22,46 +24,75 @@ class OrmPool(utils.Pool):
     Foos have the same bar_id, but you only want to pull the Bar instance from
     the db once, this allows you to easily do that
 
-    example --
+    :Example:
         bar_pool = Bar.pool(500) # keep the pool contained to the last 500 Bar instances
         for f in Foo.query.all():
             b = bar_pool[f.bar_id]
             print "Foo {} loves Bar {}".format(f.pk, b.pk)
     """
-    def __init__(self, orm_class, size=0):
-        super(OrmPool, self).__init__(size=size)
+    def __init__(self, orm_class, maxsize=0):
+        super(OrmPool, self).__init__(maxsize=maxsize)
         self.orm_class = orm_class
 
-    def create_value(self, pk):
-        return self.orm_class.query.get_pk(pk)
+    def __missing__(self, pk):
+        o = self.orm_class.query.get_pk(pk)
+        self[pk] = o
+        return o
 
 
 class Orm(object):
     """
-    this is the parent class of any model Orm class you want to create that can access the db
+    this is the parent class of any Orm child class you want to create that can access the db
 
-    example -- create a user class
+    :example:
+        from prom import Orm, Field
 
-        import prom
+        # create a simple class using standard fields
+        class Foo(Orm):
+            table_name = "<TABLE NAME>"
 
-        class User(prom.Orm):
-            table_name = "user_table_name"
+            bar = Field(int, True, max_size=512, default=0, unique=True)
+            che = prom.Field(str, True)
 
-            username = prom.Field(str, True, unique=True) # set a unique index on user
-            password = prom.Field(str, True)
-            email = prom.Field(str, True)
+            index_barche = prom.Index('bar', 'che')
 
-            index_email = prom.Index('email') # set a normal index on email
+        # create a more complex class using a field override
+        class Foo2(Orm):
+            table_name = "<TABLE NAME>"
 
-        # create a user
-        u = User(username='foo', password='awesome_and_secure_pw_hash', email='foo@bar.com')
-        u.set()
+            class bar(Field):
+                type = int
+                required = True
+                options = {
+                    "default": 0,
+                    "unique": True,
+                    "max_size": 512,
+                }
 
-        # query for our new user
-        u = User.query.is_username('foo').get_one()
-        print u.username # foo
+                def fget(self, orm, v):
+                    print("fget")
+                    return v
+
+                def iget(self, orm, v):
+                    print("iget")
+                    return v
+
+                def fset(self, orm, v):
+                    print("fset")
+                    return v
+
+                def fdel(self, orm, v):
+                    print("fdel")
+                    return v
+
+                def iquery(self, query, v):
+                    print("iquery")
+                    return v
+
+                def jsonable(self, orm, v):
+                    print("jsonable")
+                    return v
     """
-
     connection_name = ""
     """the name of the connection to use to retrieve the interface"""
 
@@ -72,20 +103,26 @@ class Orm(object):
     """the class this Orm will use for iterating through results returned from db"""
 
     _id = Field(long, True, pk=True)
-    _created = Field(datetime.datetime, True, idefault_insert=lambda *_, **__: datetime.datetime.utcnow())
-    _updated = Field(datetime.datetime, True, idefault=lambda *_, **__: datetime.datetime.utcnow())
 
-#     @_created.isetter
-#     def _created(self, val, is_update, is_modified):
-#         if not is_modified and not is_update:
-#             val = datetime.datetime.utcnow()
-#         return val
-# 
-#     @_updated.isetter
-#     def _updated(self, val, is_update, is_modified):
-#         if not is_modified:
-#             val = datetime.datetime.utcnow()
-#         return val
+    class _created(Field):
+        type = datetime.datetime
+        required = True
+        options = {
+            "aliases": ["created"],
+        }
+
+        def iset(self, orm, val):
+            return datetime.datetime.utcnow() if orm.is_insert() else val
+
+    class _updated(Field):
+        type = datetime.datetime
+        required = True
+        options = {
+            "aliases": ["updated"],
+        }
+
+        def iset(self, orm, val):
+            return datetime.datetime.utcnow()
 
     @decorators.classproperty
     def table_name(cls):
@@ -110,13 +147,13 @@ class Orm(object):
 
     @decorators.classproperty
     def query(cls):
-        """
-        return a new Query instance ready to make a db call using the child class
+        """return a new Query instance ready to make a db call using the child class
 
-        example -- fluid interface
+        :example:
+            # fluid interface
             results = Orm.query.is_foo('value').desc_bar().get()
 
-        return -- Query() -- every time this is called a new query instance is created using cls.query_class
+        :returns: Query, every time this is called a new query instance is created using cls.query_class
         """
         query_class = cls.query_class
         return query_class(orm_class=cls)
@@ -147,26 +184,14 @@ class Orm(object):
         """
         return {k:getattr(self, k, None) for k in self.schema.fields}
 
-    def __init__(self, fields=None, **fields_kwargs):
-        """Create an Orm object
-
-        :param fields: dict, the fields in a dict
-        :param **fields_kwargs: dict, if you would like to pass the fields as key=val
-        """
-        self.reset_modified()
-
-        fields = self.make_dict(fields, fields_kwargs)
-        if fields:
-            self.modify(fields)
-
     @classmethod
-    def pool(cls, size=0):
+    def pool(cls, maxsize=0):
         """
         return a new OrmPool instance
 
         return -- OrmPool -- the orm pool instance will be tied to this Orm
         """
-        return OrmPool(orm_class=cls, size=size)
+        return OrmPool(orm_class=cls, maxsize=maxsize)
 
     @classmethod
     def create(cls, fields=None, **fields_kwargs):
@@ -182,7 +207,7 @@ class Orm(object):
         return instance
 
     @classmethod
-    def populated(cls, fields=None, **fields_kwargs):
+    def hydrate(cls, fields=None, **fields_kwargs):
         """return a populated instance with the present fields
 
         NOTE -- you probably shouldn't override this method since the Query methods
@@ -193,7 +218,8 @@ class Orm(object):
         :returns: an instance of this class with populated fields
         """
         instance = cls()
-        instance.populate(fields, **fields_kwargs)
+        fields = cls.make_dict(fields, fields_kwargs)
+        instance.from_interface(fields)
         return instance
 
     @classmethod
@@ -209,9 +235,42 @@ class Orm(object):
         """
         return utils.make_dict(fields, fields_kwargs)
 
+    def __init__(self, fields=None, **fields_kwargs):
+        """Create an Orm object
+
+        :param fields: dict, the fields in a dict
+        :param **fields_kwargs: dict, if you would like to pass the fields as key=val
+        """
+        self.reset_modified()
+
+        fields = self.make_dict(fields, fields_kwargs)
+        fields = self.modify_fields(fields)
+
+        for field_name, field in self.schema.fields.items():
+            if field_name in fields:
+                field_val = fields[field_name]
+
+            else:
+                field_val = field.fdefault(self, None)
+
+            setattr(self, field_name, field_val)
+
+        self._interface_pk = None
+        self._interface_hydrate = False
+
     def fk(self, orm_class):
         """find the field value in self that is the primary key of the passed in orm_class
 
+        :example:
+            class Foo(Orm):
+                pass
+
+            class Bar(Orm):
+                foo_id = Field(Foo)
+
+            b = Bar(foo_id=1)
+            print(b.fk(Foo)) # 1
+
         :param orm_class: Orm, the fields in self will be checked until the field that
             references Orm is found, then the value of that field will be returned
         :returns: the self field value that is a foreign key references to orm_class
@@ -225,31 +284,20 @@ class Orm(object):
             self.__class__.__name__,
         ))
 
-    def populate(self, fields=None, **fields_kwargs):
-        """take the passed in fields, combine them with missing fields that should
-        be there and then run all those through appropriate methods to hydrate this
-        orm.
+    def is_hydrated(self):
+        """return True if this orm was populated from the interface/db"""
+        return self._interface_hydrate
 
-        The method replaces cls.hydrate() since it was becoming hard to understand
-        what was going on with all these methods that did things just a little bit
-        different.
+    def is_update(self):
+        """Return True if .save() will perform an interface update"""
+        pk = self._interface_pk
+        return pk is not None
 
-        This is used to completely set all the fields of self. If you just want
-        to populate certain fields, you can use the submethod populate_fields
+    def is_insert(self):
+        """Return True if .save() will perform an interface insert"""
+        return not self.is_update()
 
-        :param fields: dict, the fields in a dict that will be combined with all the
-            fields of the Orm
-        :param **fields_kwargs: dict, if you would like to pass the fields as key=val
-            this picks those up and combines them with fields
-        """
-        pop_fields = {}
-        fields = self.make_dict(fields, fields_kwargs)
-        for k in self.schema.fields.keys():
-            pop_fields[k] = fields.get(k, None)
-
-        self.populate_fields(pop_fields)
-
-    def populate_fields(self, fields):
+    def from_interface(self, fields):
         """this runs all the fields through their iget methods to mimic them
         freshly coming out of the db, then resets modified
 
@@ -268,37 +316,34 @@ class Orm(object):
         for field_name in fields.keys():
             self.reset_modified(field_name)
 
-    def depopulate(self, is_update):
+        # this marks that this was repopulated from the interface (database)
+        self._interface_pk = self.pk
+        self._interface_hydrate = True
+
+    def to_interface(self):
         """Get all the fields that need to be saved
 
         :param is_udpate: bool, True if update query, False if insert
         :returns: dict, key is field_name and val is the field value to be saved
         """
+        is_update = self.is_update()
         fields = {}
         schema = self.schema
         for k, field in schema.fields.items():
-            is_modified = k in self.modified_fields
-            orig_v = getattr(self, k)
-            v = field.isave(
-                self,
-                orig_v,
-                is_update=is_update,
-                is_modified=is_modified
-            )
+            v = field.iset(self, getattr(self, k))
+            is_modified = self.is_modified(k)
 
             # if v has a value or it's been modified (even if it doesn't have a
             # value) then we will probably want to pass the value onto the db
             if is_modified or v is not None:
-                if is_update and field.is_pk() and v == orig_v:
-                    continue
+                fields[k] = v
+
+            if v is None and field.is_required():
+                if field.is_pk() and v == self._interface_pk:
+                    fields.pop(k, None)
 
                 else:
-                    fields[k] = v
-
-        if not is_update:
-            for field_name in schema.required_fields.keys():
-                if field_name not in fields:
-                    raise KeyError("Missing required field {}".format(field_name))
+                    raise KeyError("Missing required field {}".format(k))
 
         return fields
 
@@ -307,18 +352,18 @@ class Orm(object):
         ret = True
 
         schema = self.schema
-        fields = self.depopulate(False)
-
+        fields = self.to_interface()
 
         q = self.query
-        q.set_fields(fields)
+        q.set(fields)
         pk = q.insert()
         if pk:
-            fields = q.fields
+            fields = q.fields_set.fields
             pk_name = schema.pk_name
             if pk_name:
                 fields[pk_name] = pk
-            self.populate_fields(fields)
+                self.from_interface(fields)
+            self.reset_modified()
 
         else:
             ret = False
@@ -328,20 +373,22 @@ class Orm(object):
     def update(self):
         """re-persist the updated field values of this orm that has a primary key"""
         ret = True
-        fields = self.depopulate(True)
-        q = self.query
-        q.set_fields(fields)
+        fields = self.to_interface()
 
-        pk = self.pk
+        q = self.query
+        q.set(fields)
+
+        pk = self._interface_pk
         if pk:
             q.is_field(self.schema.pk.name, pk)
 
         else:
-            raise ValueError("You cannot update without a primary key")
+            raise ValueError("Cannot update an unhydrated orm instance")
 
         if q.update():
-            fields = q.fields
-            self.populate_fields(fields)
+            fields = q.fields_set.fields
+            self.from_interface(fields)
+            self.reset_modified()
 
         else:
             ret = False
@@ -357,12 +404,7 @@ class Orm(object):
         """
         ret = False
 
-        # we will only use the primary key if it hasn't been modified
-        pk = None
-        pk_name = self.schema.pk_name
-        if pk_name not in self.modified_fields:
-            pk = self.pk
-
+        pk = self._interface_pk
         if pk:
             ret = self.update()
         else:
@@ -374,7 +416,7 @@ class Orm(object):
         """delete the object from the db if pk is set"""
         ret = False
         q = self.query
-        pk = self.pk
+        pk = self._interface_pk
         if pk:
             pk_name = self.schema.pk.name
             self.query.is_field(pk_name, pk).delete()
@@ -386,13 +428,32 @@ class Orm(object):
                 if getattr(self, field_name, None) != None:
                     self.modified_fields.add(field_name)
 
+            self._interface_pk = None
+            self._interface_hydrate = False
+
             ret = True
 
         return ret
 
-    def is_modified(self):
-        """true if a field has been changed from its original value, false otherwise"""
-        return len(self.modified_fields) > 0
+    def requery(self):
+        """Fetch this orm from the db again (ie, re-query the row from the db and
+        return a new Orm instance with the columns from that row)"""
+        pk = self._interface_pk
+        if not pk:
+            raise ValueError("Unable to refetch orm via hydrated primary key")
+        return self.query.eq_pk(pk).one()
+
+    def is_modified(self, field_name=""):
+        """true if a field, or any field, has been changed from its original value, false otherwise
+
+        :param field_name: string, the name of the field you want to check for modification
+        :returns: bool
+        """
+        if field_name:
+            ret = field_name in self.modified_fields
+        else:
+            ret = len(self.modified_fields) > 0
+        return ret
 
     def reset_modified(self, field_name=""):
         """
@@ -406,7 +467,7 @@ class Orm(object):
         """
         if field_name:
             field = self.schema.fields[field_name]
-            if not isinstance(field, ObjectField):
+            if not field.is_serialized():
                 self.modified_fields.discard(field_name)
 
         else:
@@ -414,7 +475,7 @@ class Orm(object):
 
             # compensate for us not having knowledge of certain fields changing
             for field_name, field in self.schema.normal_fields.items():
-                if isinstance(field, ObjectField):
+                if field.is_serialized():
                     self.modified_fields.add(field_name)
 
     def modify(self, fields=None, **fields_kwargs):
@@ -428,16 +489,17 @@ class Orm(object):
             this picks those up and combines them with fields
         :returns: set, all the names of the fields that were modified
         """
-        modified_fields = set()
+        modified_fields = set(self.modified_fields)
+
         fields = self.make_dict(fields, fields_kwargs)
         fields = self.modify_fields(fields)
+
         for field_name, field_val in fields.items():
             in_schema = field_name in self.schema.fields
             if in_schema:
                 setattr(self, field_name, field_val)
-                modified_fields.add(field_name)
 
-        return modified_fields
+        return self.modified_fields - modified_fields
 
     def modify_fields(self, fields):
         """In child classes you should override this method to do any default 
@@ -451,15 +513,7 @@ class Orm(object):
 
     def __setattr__(self, field_name, field_val):
         if field_name in self.schema.fields:
-            if self.schema.fields[field_name].is_pk():
-                # we mark everything as dirty because the primary key has changed
-                # and so a new row would be inserted into the db
-                self.modified_fields.add(field_name)
-                self.modified_fields.update(self.schema.normal_fields.keys())
-
-            else:
-                self.modified_fields.add(field_name)
-
+            self.modified_fields.add(field_name)
         super(Orm, self).__setattr__(field_name, field_val)
 
     def __delattr__(self, field_name):
